using System;
using System.Collections.Generic;
using System.Text;
using Ivi.Visa.Interop;
using System.IO;

namespace PXBVISACSharp
{
    public class AgilentInstrument:IDisposable
    {
        ResourceManager s_resourceManager = null;
        const string s_noError = "0,\"No error\"";
        FormattedIO488Class m_ioSession = null;
        public AgilentInstrument()
        {
            s_resourceManager = new ResourceManager();
            m_address = String.Empty;
            m_lastCommand = String.Empty;
            m_lastResult = String.Empty;
            m_lastError = String.Empty;
        }

        /// <summary>
        /// Make a connection to an instrument
        /// </summary>
        /// <param name="address">The address of the instrument</param>
        /// <param name="timeOut">The amount of time to wait when talking to the instrument</param>
        /// <remarks> Some address formats:
        ///     SCPI address = "GPIB1::19::INSTR";
        ///     LAN address = "TCPIP0::a-n5106a-00032::INSTR";
        ///     USB address = "USB0::2391::291::0000000032::0::INSTR";
        /// </remarks>
        public void Connect(string address, int timeOut)
        {
            System.Diagnostics.Trace.Assert(address != null && timeOut >= 0);
            try
            {
                try
                {
                    if (IsConnected)
                    {
                        m_ioSession.IO.Close();
                        m_ioSession = null;
                    }
                }
                catch (System.Runtime.InteropServices.COMException)
                {
                    //Swallow the exception
                }
                m_ioSession = new FormattedIO488Class();
                m_ioSession.IO = (IMessage)s_resourceManager.Open(address, AccessMode.NO_LOCK, 15000, "");
                m_ioSession.IO.Timeout = timeOut;
                m_ioSession.WriteString("*IDN?", true);
                m_id = m_ioSession.ReadString();
                m_ioSession.WriteString("*CLS", true);
                m_ioSession.WriteString("*RST", true);
                Console.WriteLine(String.Format("{0} connected at address {1}", m_id, address));
                
            }
            catch (System.Runtime.InteropServices.COMException ex)
            {
                m_ioSession = null;
                m_address = String.Empty;
                m_id = String.Empty;
                Console.WriteLine(String.Format("Failed to connect to instrument at address {0}.  Error: {1}", address, ex.Message));
            }
        }

        public bool IsConnected
        {
            get { return (m_ioSession != null); }
        }

        string m_address;
        //The instrument address
        public string Address
        {
            get { return m_address; }
        }

        string m_id;
        //The instrument ID
        public string ID
        {
            get { return m_id; }
        }

        string m_lastError;
        public string LastError
        {
            get { return m_lastError; }
        }

        string m_lastResult;
        public string LastResult
        {
            get { return m_lastResult; }
        }

        string m_lastCommand;
        public string LastCommand
        {
            get { return m_lastCommand; }
        }

        /// <summary>
        /// Send a SCPI command to set a setting
        /// </summary>
        /// <param name="settingCommand">The command to modify some setting's value</param>
        /// <remarks>Checks LastError for any error</remarks>
        public void Write(string settingCommand)
        {
            Console.WriteLine(String.Format("Send to {0} instrument {1}", m_id, settingCommand));
            m_lastCommand = settingCommand;
            if (m_ioSession == null) return;
            m_ioSession.WriteString(settingCommand,true);
            ReadError();
            m_ioSession.IO.Clear();
            if (m_lastError != String.Empty)
            {
                Console.WriteLine(String.Format("Last Error {0}", m_lastError));
            }
            else
            {
                Console.WriteLine("No Error");
            }
        }

        /// <summary>
        /// Send a SCPI command to query some setting
        /// </summary>
        /// <param name="gettingCommand">The command to query some setting's value</param>
        /// <returns>the retrived value</returns>
        /// <remarks>If empty string is returned, check LastError for error</remarks>
        public string Read(string gettingCommand)
        {
            if (m_ioSession == null) return String.Empty;
            m_lastCommand = gettingCommand;
            m_ioSession.WriteString(gettingCommand,true);
            string result = String.Empty;
            try
            {
                result = m_ioSession.ReadString();
            }
            catch (System.Runtime.InteropServices.COMException)
            {
                result = String.Empty;
            }
            ReadError();
            m_ioSession.IO.Clear();
            m_lastResult = result;
            return result;
        }

        #region private functions
        
        private void ReadError()
        {
            if (m_ioSession == null) return;
            m_lastError = String.Empty;
            string errMsg = String.Empty;
            do
            {
                m_ioSession.WriteString("SYST:ERR?",true);
                errMsg = m_ioSession.ReadString().Trim();
                if (!errMsg.Trim().StartsWith("0"))
                {
                    m_lastError = m_lastError + errMsg;
                }
            } while (!errMsg.Trim().StartsWith("0"));
            m_ioSession.IO.Clear();
        }

        #endregion private functions

        #region IDisposable Members

        public void Dispose()
        {
            if (m_ioSession != null) m_ioSession.IO.Close();
        }

        #endregion
    }
}
